@@ -7,7 +7,67 @@
 
     Copyright (C) 1999-2006 University of Oxford  */
 
-/*  CCOPYRIGHT  */
+/*  Part of FSL - FMRIB's Software Library
+    http://www.fmrib.ox.ac.uk/fsl
+    fsl@fmrib.ox.ac.uk
+    
+    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
+    Imaging of the Brain), Department of Clinical Neurology, Oxford
+    University, Oxford, UK
+    
+    
+    LICENCE
+    
+    FMRIB Software Library, Release 4.0 (c) 2007, The University of
+    Oxford (the "Software")
+    
+    The Software remains the property of the University of Oxford ("the
+    University").
+    
+    The Software is distributed "AS IS" under this Licence solely for
+    non-commercial use in the hope that it will be useful, but in order
+    that the University as a charitable foundation protects its assets for
+    the benefit of its educational and research purposes, the University
+    makes clear that no condition is made or to be implied, nor is any
+    warranty given or to be implied, as to the accuracy of the Software,
+    or that it will be suitable for any particular purpose or for use
+    under any specific conditions. Furthermore, the University disclaims
+    all responsibility for the use which is made of the Software. It
+    further disclaims any liability for the outcomes arising from using
+    the Software.
+    
+    The Licensee agrees to indemnify the University and hold the
+    University harmless from and against any and all claims, damages and
+    liabilities asserted by third parties (including claims for
+    negligence) which arise directly or indirectly from the use of the
+    Software or the sale of any products based on the Software.
+    
+    No part of the Software may be reproduced, modified, transmitted or
+    transferred in any form or by any means, electronic or mechanical,
+    without the express permission of the University. The permission of
+    the University is not required if the said reproduction, modification,
+    transmission or transference is done without financial return, the
+    conditions of this Licence are imposed upon the receiver of the
+    product, and all original and amended source code is included in any
+    transmitted product. You may be held legally responsible for any
+    copyright infringement that is caused or encouraged by your failure to
+    abide by these terms and conditions.
+    
+    You are not permitted under this Licence to use this Software
+    commercially. Use for which any financial return is received shall be
+    defined as commercial use, and includes (1) integration of all or part
+    of the source code or the Software into a product for sale or license
+    by or on behalf of Licensee to third parties or (2) use of the
+    Software or any derivative of it for research with the final aim of
+    developing software products for sale or license to a third party or
+    (3) use of the Software or any derivative of it for research with the
+    final aim of developing non-software products for sale or license to a
+    third party, or (4) use of the Software to provide any service to an
+    external organisation for which payment is received. If you are
+    interested in using the Software commercially, please contact Isis
+    Innovation Limited ("Isis"), the technology transfer company of the
+    University, to negotiate a licence. Contact details are:
+    innovation@isis.ox.ac.uk quoting reference DE/1112. */
 
 #include <iostream>
 #include <string>
@@ -668,6 +728,10 @@ int main(int argc, char *argv[]) {
 
   //the real program
 
+  volume<float> testvol;
+  
+  if (read_volume(testvol,in.c_str())<0)  return -1;
+
   double xarg = 0, yarg = 0, zarg = 0;
   if (centerarg.set())
     {
@@ -684,14 +748,15 @@ int main(int argc, char *argv[]) {
 	  xarg = (double) centerarg.value(0);
 	  yarg = (double) centerarg.value(1);
 	  zarg = (double) centerarg.value(2);
+	  ColumnVector v(4);
+	  v << xarg << yarg << zarg << 1.0;
+	  v = testvol.niftivox2newimagevox_mat() * v;
+	  xarg = v(1);  yarg = v(2);  zarg = v(3);
 	}
     }
   
   const double bet_main_parameter = pow(fractional_threshold.value(), .275);
   
-  volume<float> testvol;
-  
-  if (read_volume(testvol,in.c_str())<0)  return -1;
 
   // 2D kludge (worked for bet, but not here in bet2, hohum)
   if (testvol.xsize()*testvol.xdim()<20) testvol.setxdim(200);
