@@ -6,7 +6,8 @@ fudge=$3
 bi=$4
 njumps=$5
 sampleevery=$6
-slice=$7
+model=$7
+slice=$8
 
 slicezp=`${FSLDIR}/bin/zeropad $slice 4`
 
@@ -15,5 +16,5 @@ ${FSLDIR}/bin/xfibres\
  --mask=$subjdir/nodif_brain_mask_slice_$slicezp\
  -b $subjdir/bvals -r $subjdir/bvecs\
  --forcedir --logdir=$subjdir.bedpostX/diff_slices/data_slice_$slicezp\
- --fudge=$fudge --nj=$njumps --bi=$bi --se=$sampleevery --upe=24 --nfibres=$nfibres > $subjdir.bedpostX/logs/log$slicezp  && echo Done
+ --fudge=$fudge --nj=$njumps --bi=$bi --model=$model --nonlinear --se=$sampleevery --upe=24 --nfibres=$nfibres  > $subjdir.bedpostX/logs/log$slicezp  && echo Done
 
