@@ -100,29 +100,35 @@ class probtrackxOptions {
   Option<bool> s2tout;
   FmribOption<bool> matrix1out;
   FmribOption<bool> matrix2out;
+  FmribOption<bool> matrix3out;
+  FmribOption<string> maskmatrix3;
   FmribOption<bool> maskmatrixout;
   Option<string> outfile;
   Option<string> rubbishfile;
   Option<string> stopfile;
-  Option<string> prefdirfile;
+  FmribOption<string> prefdirfile;
   Option<string> seeds_to_dti;
+  Option<string> dti_to_seeds;
   FmribOption<string> skipmask;
   Option<string> seedref;
   Option<string> mask2;
   Option<string> waypoints;
   Option<bool> network;
   Option<string> meshfile;
+  FmribOption<string> meshspace;
   FmribOption<string> lrmask;
   Option<string> logdir; 
   Option<bool> forcedir;
   Option<int> nparticles;
   Option<int> nsteps;
+  Option<float> distthresh;
   Option<float> c_thr;
   FmribOption<float> fibthresh;
+  Option<bool> sampvox;
   Option<float> steplength;
   Option<bool> loopcheck;
   Option<bool> usef;
-  Option<bool> randfib;
+  Option<int> randfib;
   Option<int> fibst;
   Option<bool> modeuler;
   Option<int> rseed;
@@ -155,58 +161,68 @@ class probtrackxOptions {
 
  inline probtrackxOptions::probtrackxOptions() :
   verbose(string("-V,--verbose"), 0, 
-	  string("verbose level, [0-2]"), 
+	  string("Verbose level, [0-2]"), 
 	  false, requires_argument),
    help(string("-h,--help"), false,
-	string("display this message"),
+	string("Display this message"),
 	false, no_argument),
    basename(string("-s,--samples"), string("merged"),
-	       string("basename for samples files"),
+	       string("Basename for samples files"),
 	       true, requires_argument),  
    maskfile(string("-m,--mask"), string("mask"),
 	    string("Bet binary mask file in diffusion space"),
 	    true, requires_argument),
    seedfile(string("-x,--seed"), string("Seed"),
-	    string("Seed volume, or voxel, or ascii file with multiple volumes"),
+	    string("Seed volume, or voxel, or ascii file with multiple volumes, or freesurfer label file"),
 	    true, requires_argument),
    mode(string("--mode"), string(""),
-	string("use --mode=simple for single seed voxel"),
+	string("\tUse --mode=simple for single seed voxel"),
 	    false, requires_argument),
   targetfile(string("--targetmasks"), string("cmasks"),
 	    string("File containing a list of target masks - required for seeds_to_targets classification"),
 	    false, requires_argument),
   simpleout(string("--opd"), false,
-	    string("output path distribution"),
+	    string("\tOutput path distribution"),
 	    false, no_argument), 
   pathdist(string("--pd"), false,
-	   string("Correct path distribution for the length of the pathways"),
+	   string("\tCorrect path distribution for the length of the pathways"),
 	   false, no_argument), 
   s2tout(string("--os2t"), false,
-	 string("output seeds to targets"),
+	 string("\tOutput seeds to targets"),
 	 false, no_argument),
   matrix1out(string("--omatrix1"), false,
-	  string("output matrix1"),
+	  string("Output matrix1"),
 	  false, no_argument), 
   matrix2out(string("--omatrix2"), false,
-	  string("output matrix2"),
+	  string("Output matrix2"),
 	  false, no_argument), 
+  matrix3out(string("--omatrix3"), false,
+	  string("Output matrix3 (NxN connectivity matrix)"),
+	  false, no_argument), 
+  maskmatrix3(string("--mask3"), "",
+	  string("Mask used for NxN connectivity matrix"),
+	  false, requires_argument), 
   maskmatrixout(string("--omaskmatrix"), false,
-		string("output maskmatrix"),
+		string("Output maskmatrix"),
 		false, no_argument), 
-   outfile(string("-o,--out"), string(""),
-	   string("Output file (only for single seed voxel mode)"),
+   outfile(string("-o,--out"), string("fdt_paths"),
+	   string("Output file (default='fdt_paths')"),
 	   false, requires_argument),
    rubbishfile(string("--avoid"), string(""),
-	       string("Reject pathways passing through locations given by this mask"),
+	       string("\tReject pathways passing through locations given by this mask"),
 	       false, requires_argument),
    stopfile(string("--stop"), string(""),
-	       string("Stop tracking at locations given by this mask file"),
+	       string("\tStop tracking at locations given by this mask file"),
 	       false, requires_argument),
    prefdirfile(string("--prefdir"), string(""),
-	       string("prefered orientation preset in a 4D mask"),
+	       string("Prefered orientation preset in a 4D mask"),
 	       false, requires_argument),
    seeds_to_dti(string("--xfm"), string(""),
-		string("Transform Matrix taking seed space to DTI space default is to use the identity"),false, requires_argument),
+		string("\tTransform taking seed space to DTI space (either FLIRT matrix or FNIRT warpfield) - default is identity"),
+		false, requires_argument),
+   dti_to_seeds(string("--invxfm"), string(""),
+		string("Transform taking DTI space to seed space (compulsory when using a warpfield for seeds_to_dti)"),
+		false, requires_argument),
    skipmask(string("--no_integrity"), string(""),
 	   string("no explanation needed"),
 	   false, requires_argument),
@@ -214,7 +230,7 @@ class probtrackxOptions {
 	 string("Reference vol to define seed space in simple mode - diffusion space assumed if absent"),
 	 false, requires_argument),
   mask2(string("--mask2"), string(""),
-	 string("second mask in twomask_symm mode."),
+	 string("\tSecond mask in twomask_symm mode."),
        false, requires_argument),
  waypoints(string("--waypoints"), string(""),
 	 string("Waypoint mask or ascii list of waypoint masks - only keep paths going through ALL the masks"),
@@ -223,13 +239,16 @@ class probtrackxOptions {
 	 string("Activate network mode - only keep paths going through at least one seed mask (required if multiple seed masks)"),
        false, no_argument),
    meshfile(string("--mesh"), string(""),
-	 string("Freesurfer-type surface descriptor (in ascii format)"),
+	 string("\tFreesurfer-type surface descriptor (in ascii format)"),
+       false, requires_argument),
+   meshspace(string("--meshspace"), string("freesurfer"),
+	 string("Mesh reference space - either 'freesurfer' (default) or 'caret'"),
        false, requires_argument),
   lrmask(string("--lrmask"), string(""),
-	 string("low resolution binary brain mask for stroring connectivity distribution in matrix2 mode"),
+	 string("Low resolution binary brain mask for storing connectivity distribution in matrix2 mode"),
        false, requires_argument),
-  logdir(string("--dir"), string(""),
-	    string("Directory to put the final volumes in - code makes this directory"),
+  logdir(string("--dir"), string("logdir"),
+	    string("\tDirectory to put the final volumes in - code makes this directory - default='logdir'"),
 	    false, requires_argument),
   forcedir(string("--forcedir"), false,
 	 string("Use the actual directory name given - i.e. don't add + to make a new directory"),
@@ -240,38 +259,44 @@ class probtrackxOptions {
    nsteps(string("-S,--nsteps"), 2000,
 	    string("Number of steps per sample - default=2000"),
 	    false, requires_argument),
+   distthresh(string("--distthresh"), 0,
+	    string("Discards samples shorter than this threshold (in mm - default=0)"),
+	    false, requires_argument),
    c_thr(string("-c,--cthr"), 0.2, 
 	 string("Curvature threshold - default=0.2"), 
 	 false, requires_argument),
   fibthresh(string("--fibthresh"), 0.01, 
-	    string("volume fraction before subsidary fibre orientations are considered - default=0.01"), 
+	    string("Volume fraction before subsidary fibre orientations are considered - default=0.01"), 
 	 false, requires_argument),
+  sampvox(string("--sampvox"), false, 
+	    string("Sample random points within seed voxels"), 
+	 false, no_argument),
    steplength(string("--steplength"), 0.5, 
-	 string("steplength in mm - default=0.5"), 
+	 string("Steplength in mm - default=0.5"), 
 	 false, requires_argument),
    loopcheck(string("-l,--loopcheck"), false, 
-	 string("perform loopchecks on paths - slower, but allows lower curvature threshold"), 
+	 string("Perform loopchecks on paths - slower, but allows lower curvature threshold"), 
 	 false, no_argument),
    usef(string("-f,--usef"), false, 
 	 string("Use anisotropy to constrain tracking"), 
 	 false, no_argument),
-  randfib(string("--randfib"), false, 
-	 string("Select randomly from one of the fibres"), 
-	 false, no_argument),
+  randfib(string("--randfib"), 0, 
+	 string("Default 0. Set to 1 to randomly sample initial fibres (with f > fibthresh). \n                        Set to 2 to sample in proportion fibres (with f>fibthresh) to f. \n                        Set to 3 to sample ALL populations at random (even if f<fibthresh)"), 
+	 false, requires_argument),
   fibst(string("--fibst"),1, 
-	 string("Force a starting fibre for tracking - default=1, i.e. first fibre orientation"), 
+	 string("\tForce a starting fibre for tracking - default=1, i.e. first fibre orientation. Only works if randfib==0"), 
 	 false, requires_argument),
   modeuler(string("--modeuler"), false, 
 	   string("Use modified euler streamlining"), 
 	   false, no_argument),
   rseed(string("--rseed"), 12345,
-	string("Random seed"),
+	string("\tRandom seed"),
 	false, requires_argument), 
-  seedcountastext(string("--seedcountastext"), false,
+  seedcountastext(string("--s2tastext"), false,
 		  string("Output seed-to-target counts as a text file (useful when seeding from a mesh)"),
 		  false, no_argument), 
   splitmatrix2(string("--splitmatrix2"), false,
-		  string("split matrix 2 (in case it is too big)"),
+		  string("Split matrix 2 along seed dimension (in case it is too large)"),
 		  false, no_argument), 
    options("probtrackx","probtrackx -s <basename> -m <maskname> -x <seedfile> -o <output> --targetmasks=<textfile>\n probtrackx --help\n")
    {
@@ -290,6 +315,7 @@ class probtrackxOptions {
        options.add(waypoints);
        options.add(network);
        options.add(meshfile);
+       options.add(meshspace);
        options.add(lrmask);
        options.add(seedref);
        options.add(logdir); 
@@ -299,16 +325,21 @@ class probtrackxOptions {
        options.add(s2tout);
        options.add(matrix1out);
        options.add(matrix2out);
+       options.add(matrix3out);
+       options.add(maskmatrix3);
        options.add(maskmatrixout);
        options.add(outfile);
        options.add(rubbishfile);
        options.add(stopfile);
        options.add(prefdirfile);
        options.add(seeds_to_dti);
+       options.add(dti_to_seeds);
        options.add(nparticles);
        options.add(nsteps);
+       options.add(distthresh);
        options.add(c_thr);
        options.add(fibthresh);
+       options.add(sampvox);
        options.add(steplength);
        options.add(loopcheck);
        options.add(usef);
