@@ -91,6 +91,11 @@ void nmasks()
   else
     read_masks(masknames,opts.seedfile.value());
 
+  if(opts.s2tout.value()){
+    cerr << "Seed_to_target not available in multiple seed tractography" << endl;
+    exit(0);
+  }
+
   for(unsigned int m=0;m<masknames.size();m++){
     read_volume(tmpvol,masknames[m]);
     tmpvol=NEWIMAGE::abs(tmpvol);
@@ -98,7 +103,7 @@ void nmasks()
     seeds.push_back(tmpvol);
   }
 
-  Streamliner stline;
+  Streamliner stline(seeds[0]);
   Counter counter(seeds[0],stline);
   counter.initialise();
   Seedmanager seedmanager(counter);
