@@ -196,11 +196,12 @@ int FslFileType(const char* fname)
   flen = strlen(fname);
   if (flen<5) return retval;  /* smallest name + extension is a.nii */
   if (strcmp(fname + flen - 4,".nii")==0)  retval=FSL_TYPE_NIFTI;
-  if (strcmp(fname + flen - 7,".nii.gz")==0)  retval=FSL_TYPE_NIFTI_GZ;
   if (strcmp(fname + flen - 4,".mnc")==0)  retval=FSL_TYPE_MINC;
-  if (strcmp(fname + flen - 7,".mnc.gz")==0)  retval=FSL_TYPE_MINC;
   if (strcmp(fname + flen - 4,".hdr")==0)  retval=FSL_TYPE_NIFTI_PAIR;
   if (strcmp(fname + flen - 4,".img")==0)  retval=FSL_TYPE_NIFTI_PAIR;
+  if ((retval==-1) && (flen<8)) return retval; /* small name + ext.gz is a.nii.gz */
+  if (strcmp(fname + flen - 7,".mnc.gz")==0)  retval=FSL_TYPE_MINC;
+  if (strcmp(fname + flen - 7,".nii.gz")==0)  retval=FSL_TYPE_NIFTI_GZ;
   if (strcmp(fname + flen - 7,".hdr.gz")==0)  retval=FSL_TYPE_NIFTI_PAIR_GZ;
   if (strcmp(fname + flen - 7,".img.gz")==0)  retval=FSL_TYPE_NIFTI_PAIR_GZ;
   if ( (retval==FSL_TYPE_NIFTI_PAIR) || (retval==FSL_TYPE_NIFTI_PAIR_GZ) ) {
@@ -285,10 +286,10 @@ char *FslMakeBaseName(const char *fname)
   basename = nifti_makebasename(fname);
   blen = strlen(basename);
 #ifdef HAVE_ZLIB
-  if (strcmp(basename + blen-7,".mnc.gz") == 0) 
+  if ((blen>7) && (strcmp(basename + blen-7,".mnc.gz") == 0))
     { basename[blen-7]='\0'; return basename; }
 #endif
-  if (strcmp(basename + blen-4,".mnc") == 0) 
+  if ((blen>4) && (strcmp(basename + blen-4,".mnc") == 0))
     { basename[blen-4]='\0'; return basename; }
   return basename;
 }
@@ -828,7 +829,7 @@ void* FslReadAllVolumes(FSLIO* fslio, char* filename)
  */
 size_t FslReadVolumes(FSLIO *fslio, void *buffer, size_t nvols)
 {
-  int volbytes;
+  size_t volbytes;
   size_t retval=0;
   if (fslio==NULL)  FSLIOERR("FslReadVolumes: Null pointer passed for FSLIO");
   if (znz_isnull(fslio->fileptr))  FSLIOERR("FslReadVolumes: Null file pointer");
@@ -2072,6 +2073,7 @@ int FslClose(FSLIO *fslio)
     if (znz_isnull(hptr)) {     
       fprintf(stderr,"Error:: Could not write origin data to header file %s.\n",
               fslio->niftiptr->fname);
+      free(hdr);
       return -1;
     };
     
